/*******************************************************************************
 mp4_process.h -

 Copyright (C) 2007-2020 CodeShop B.V.
 http://www.code-shop.com

 For licensing see the LICENSE file
******************************************************************************/

#ifndef MP4_PROCESS_H_AKW
#define MP4_PROCESS_H_AKW

#include "mod_streaming_export.h"
#include "mp4_error.h"
#include <inttypes.h>

#ifdef __cplusplus
extern "C" {
#endif

// Following section carries a global init/exit function which should be
// called as first and as last function of the API (before and after
// anything else)

typedef struct mp4_global_context_t mp4_global_context_t;

MP4_DLL_EXPORT extern
mp4_global_context_t* libfmp4_global_init();

MP4_DLL_EXPORT extern
void libfmp4_global_exit(mp4_global_context_t* gctx);

MP4_DLL_EXPORT extern
char const* libfmp4_load_license(mp4_global_context_t* gctx,
                                 char const* src,
                                 char const* version,
                                 char const* license);

MP4_DLL_EXPORT extern
char const* libfmp4_print_license(mp4_global_context_t* gctx,
                                  char const* src,
                                  char const* version,
                                  char const* license);

// API data types

typedef struct buckets_t buckets_t;
typedef struct mp4_split_options_t mp4_split_options_t;
typedef struct mp4_process_context_t mp4_process_context_t;

// Hook to override the default AES decoder. When an encrypted fragment is read
// then the fragment can be decrypted before further processing (and thus
// allowing to have e.g. Common Encrypted content stored on disk, but apply a
// different DRM system (e.g. HLS AES or HDS FLXS) to the final output.
//
// The type is:
// 1 = AES_CTR (AES 128-bit CTR) or
// 2 = AES_CBC (AES 128-bit CBC).
//
// The 128-bit KID uniquely identifies the CEK to be used.
//
// When the requested aes_decoder cannot be created (e.g. you don't have the
// associated content_key, or the requested AES type is not supported by your
// hook) you must return NULL.

typedef struct fmp4_aes_decoder* (*create_aes_decoder_t)(void* context, uint32_t type, uint8_t const kid[16]);
typedef struct fmp4_aes_encoder* (*create_aes_encoder_t)(void* context, uint32_t type, uint8_t const kid[16]);

// The I/O handler used. By default this is set to a handler that provides the
// built-in file (memory mapped) and http (through libcurl) functions.
//
// If you want to use your own I/O functions, you can override the
// create_handler_io_. In your function you can check the given URL and if you
// do not want to handle it, just pass it on the previous handler. This allows
// you to override specifically for specific URLs (e.g. only http).
//
// Reference:
// mp4_handler_io.hpp: The abstract base class for I/O.
// mp4_handler_io_file.cpp: Implementation for reading/writing files from disk.
// mp4_handler_io_http.cpp: Implementation for reading URLs from http.

typedef struct fmp4_handler_io_t* (*create_handler_io_t)(mp4_process_context_t* context, char const* url, int flags);

// Function type for implementing a download handler in a webserver module.
// Returns the HTTP status (200, etc) if the GET request succeeds.
// Returns AP_FILTER_ERROR when the download handler failed internally.
typedef int (*download_t)(void* context,
                          char const* url,
                          uint64_t* offset,
                          uint32_t* len,
                          uint64_t* size,
                          buckets_t* buckets);

// Requests the server variable 'name'. The pointer to the value must be
// returned in *value.  Returns the size of the string.
// E.g. "REQUEST_SCHEME", "HTTP_HOST", "REQUEST_URI"

typedef int (*get_server_variable_callback_t)(void* context, char const* name, char const** value);

// Callback for error logging.
// level is one of FMP4_LOG_DEBUG, FMP4_LOG_WARNING, FMP4_LOG_ERROR.

typedef enum
{
  // error conditions
  FMP4_LOG_ERROR,

  // warning conditions
  FMP4_LOG_WARNING,

  // normal, but significant condition.
  // We don't use this, since in Apache the notice cannot be suppressed and thus
  // are always logged.
  FMP4_LOG_NOTICE_DO_NOT_USE,

  // informational
  FMP4_LOG_INFO,

  // debug-level messages
  FMP4_LOG_DEBUG,

  // debug-level messages
  FMP4_LOG_TRACE

} fmp4_log_level_t;

typedef void (*log_error_callback_t)(void* context, fmp4_log_level_t level, char const* str);

typedef void (*progress_callback_t)(void* context, char const* str, int done);

struct pool_t;

struct mp4_process_context_t
{
  // program wide settings
  mp4_global_context_t* global_context;

  // The path/url to the source file.
  char const* filename_;

  // The output file / request url.
  // Use to create relative links; DO NOT WRITE to this from mp4_process()!
  char const* output_url_;

  // The filesize of the source file, or UINT64_MAX when unknown.
//uint64_t filesize_;

  // Verbosity level (0=none)
  fmp4_log_level_t verbose_;

  // The output buckets
  buckets_t* buckets_;

  // Options passed in the query parameters.
  struct mp4_split_options_t* options_;

  fmp4_result result_;

  // In case of an error, holds additional information.
  char result_text_[256];

  // The command-line tool is less strict in its permissions.
  // 1. Allows for progressive download even when it isn't specifically enabled
  //    in the server manifest.
  // 2. Allow all POST actions. I.e. if this flag is set all actions are allowed
  //    for which normally the is_post_ flag has to be set.
  uint32_t no_restrictions_;

  // Log warnings about any elements ignored by XML parser classes.
  uint32_t show_xml_ignored_warnings_;

  // callbacks

  // Override the default AES decoder. The default is to use the internal
  // implementation which looks for key information in cpix::content_keys_t.
  create_aes_decoder_t create_aes_decoder_;
  void* create_aes_decoder_context_;

  // Override the default AES encoder. The default is to use the internal
  // implementation which looks for key information in cpix::content_keys_t.
  create_aes_encoder_t create_aes_encoder_;
  void* create_aes_encoder_context_;

  //
  create_handler_io_t create_handler_io_;
  download_t download_;

  //
  get_server_variable_callback_t get_server_variable_callback_;
  void* get_server_variable_context_;

  //
  log_error_callback_t log_error_callback_;
  void* log_error_context_;

  char progress_bar_[80];
  progress_callback_t progress_callback_;

  struct redirector_t* stderr_redirector_;

  struct pool_t* pool_;

  // The base path of the output (only used when packaging F4F files directly
  // with mp4split, so it outputs the F4X files to the same directory).
  //char const* out_base_path_;

  // Prefer static files over dynamically generated files (e.g. for client
  // manifest files). The @clientManifestRelativePath in the server manifest
  // specifies its location.
  uint32_t prefer_static_;

  // Pass through MP4 fragments directly for Smooth Streaming. The Origin maps
  // the given time in the media request to a byte-range.
  uint32_t iss_pass_through_;

  // Update the @timeShiftBufferDepth dynamically according to the available
  // time window (with a maximum of dvr_window_length).
  uint32_t dynamic_time_shift_buffer_depth_;

  // Set to true when called from the webserver module and the HTTP method
  // equals POST.
  uint32_t is_post_;

  // (S3) Authorization
  char const* secret_key_;
  char const* access_key_;
  char const* s3_region_;
  int s3_use_headers_;

  // Transcode origin URL.
  //   - When set to a nullptr then Unified Origin has no transcoding
  //     functionality.
  //   - When set to "http://localhost/", the Unified Origin uses its internal
  //     transcoding functionality.
  //   - Otherwise Unified Origin proxies the transcoding request to the given
  //     URL.
  char const* transcode_proxy_pass_;

  // Temporary field (we hope): use streaming transcode?
  // (Intentionally keeping this undocumented for now)
  int streaming_transcode_;

  // Pipeline configuration for transcode server
  char const* pipeline_config_;

  // time when this context was created (in microseconds since 1970). Used e.g.
  // to show the relative timing in the log.
  uint64_t epoch_;
};

// The mp4_process function is the main entry point for the libfmp4 library.
//
// An example for retrieving the client manifest file would be as follows:
//
// char const* filename = "/var/www/videos/video.ism";
// char const* args = "file=Manifest";
//
// Create and initialize the context structure.
// mp4_process_context_t context;
// mp4_process_context_init(&context);
//
// Set the filename/url to process
// context.filename_ = filename;
//
// Parse the options
// mp4_split_options_set(context.options_, args, strlen(args));
//
// Call the library
// int http_status = mp4_process(&context);
//
// Check for an error
// if(http_status != 200)
// {
//   throw std::runtime_error(context.result_text_);
// }
//
// The output is stored in the buckets.
//
// bucket_t* bucket = buckets->bucket_;
// if(bucket)
// {
//   ... read memory/file buckets ...
// }
//
// Cleanup.
// mp4_process_context_exit(&context);
//

MP4_DLL_EXPORT extern void
mp4_process_context_init(mp4_process_context_t* context,
                         mp4_global_context_t* global_context);

MP4_DLL_EXPORT extern void
mp4_process_context_exit(mp4_process_context_t* context);

// the entry point for all 'streaming' functions. I.e. anything that takes
// a virtual-file part.
MP4_DLL_EXPORT extern int mp4_process(mp4_process_context_t* context);

MP4_DLL_EXPORT extern void
mp4_process_context_reset(mp4_process_context_t* context);

MP4_DLL_EXPORT extern void
mp4_process_context_redirect_stderr(mp4_process_context_t* context);

#ifdef __cplusplus
} /* extern C definitions */
#endif

#endif // MP4_PROCESS_H_AKW

// End Of File

